-- file: IP_packets_out.vhd
-- simulation tool to read input files and send IP packets to user application with proper signals emulating IP wrapper
-- Jing Lu jl1@arl.wustl.edu
-- Washington University
-- Applied Research Lab
-- Created: 8/12/03

-------------------------------------------------------------------------------
-- Modified 
--
-- 
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_textio.ALL;
USE std.textio.ALL;

ENTITY IP_packets_out IS
  generic (
    infile : string := "input_IP.DAT");  -- input file
  PORT (
    clk             : IN  STD_LOGIC;
    reset_l         : IN  STD_LOGIC;
    d_out_appl      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0); -- data to appl
    dataEn_out_appl : OUT std_logic;  -- data enable
    sof_out_appl    : OUT std_logic;  -- start of frame
    soIP_out_appl   : OUT std_logic;  -- start of IP header
    eof_out_appl    : OUT std_logic;  -- end of frame
    sop_out_appl    : OUT std_logic;  -- start of payload
    tca_out_appl    : IN  std_logic);   -- congestion control

END IP_packets_out;


ARCHITECTURE behavioral OF IP_packets_out IS

  CONSTANT period : time := 50 ns;
  file in_file    : text open read_mode is infile;
  
  constant ATM_header_1 : std_logic_vector(31 downto 0) := x"11111111";
  constant ATM_header_2 : std_logic_vector(31 downto 0) := x"22222222";
  constant AAL5_trailer_1 : std_logic_vector(31 downto 0) := x"33333333";
  constant AAL5_trailer_2 : std_logic_vector(31 downto 0) := x"44444444";

BEGIN  -- behavioral

  read_process : PROCESS
    
    VARIABLE line_in : LINE;
    VARIABLE a       : STD_LOGIC_VECTOR(31 DOWNTO 0);
    VARIABLE s       : STRING(8 DOWNTO 1) := "xxxxxxxx";
    
    variable IHL : integer;
    variable total_length_wd : integer;    
    variable payload_count : integer;
    variable total_length_byte : std_logic_vector(15 downto 0);
    
    
  BEGIN
    WAIT UNTIL (clk'event AND clk = '1');
    dataEn_out_appl <= '0';

    if (reset_l = '0') then
      d_out_appl <= (others => '0');
      dataEn_out_appl <= '0';
      sof_out_appl <= '0';     
      soIP_out_appl <= '0';
      eof_out_appl <= '0';
      sop_out_appl <= '0';
      s := "xxxxxxxx";
      
    elsif (reset_l = '1' and not endfile(in_file)) THEN
      d_out_appl <= (others => '0');
      dataEn_out_appl <= '0';
      sof_out_appl <= '0';     
      soIP_out_appl <= '0';
      eof_out_appl <= '0';
      sop_out_appl <= '0';
      s := "xxxxxxxx";
      
      readline(in_file, line_in);       -- read a line from file
      assert not endfile(in_file) report "Last Line Read" severity NOTE;
      read(line_in, s);

      IF (s = "wait1000") THEN
        FOR k IN 0 TO 1000 LOOP
          WAIT UNTIL (clk'EVENT AND clk = '1');
        END LOOP;
      ELSIF (s = "wait_100") THEN
        FOR k IN 0 TO 100 LOOP
          WAIT UNTIL (clk'EVENT AND clk = '1');
        END LOOP;     
      ELSIF (s = "wait__10") THEN
        FOR k IN 0 TO 10 LOOP
          WAIT UNTIL (clk'EVENT AND clk = '1');
        END LOOP;          
      ELSIF (s = "wait___5") THEN
        FOR k IN 0 TO 5 LOOP
          WAIT UNTIL (clk'EVENT AND clk = '1');
        END LOOP;          
      ELSIF (s = "IPpacket") THEN     
        while (tca_out_appl = '0') loop
          WAIT UNTIL (clk'event AND clk = '1');
        end loop;
        
        sof_out_appl <= '1';
        d_out_appl <= ATM_header_1;
        wait until (clk'event and clk = '1');
        
        sof_out_appl <= '0';
        d_out_appl <= ATM_header_2;
        wait until (clk'event and clk = '1');
                
        readline(in_file, line_in);
        hread(line_in, a);
        IHL := conv_integer(a(27 downto 24)); 
        total_length_byte := a(15 downto 0) + 3;
        total_length_wd := conv_integer(total_length_byte(15 downto 2));  
        payload_count := total_length_wd - IHL;   
        
        soIP_out_appl <= '1';
        d_out_appl <= a;
        dataEn_out_appl <= '1';
        IHL := IHL - 1;
        wait until (clk'event and clk = '1');
        soIP_out_appl <= '0';
        
        while IHL > 1 loop
          readline(in_file, line_in);
          hread(line_in, a);
          d_out_appl <= a;
          IHL := IHL - 1;
          wait until (clk'event and clk = '1');
        end loop;
        
        d_out_appl <= (others => '0');
        dataEn_out_appl <= '0';
        wait until (clk'event and clk = '1');
        
        
        readline(in_file, line_in);
        hread(line_in, a);
        d_out_appl <= a;
        dataEn_out_appl <= '1';
        IHL := IHL - 1;
        if payload_count = 0 then
          eof_out_appl <= '1';
        end if;    
        wait until (clk'event and clk = '1');
        eof_out_appl <= '0';
--        dataEn_out_appl <= '0';
        
        if payload_count > 1 then
          sop_out_appl <= '1';
          readline(in_file, line_in);
          hread(line_in, a);
          dataEn_out_appl <= '1';
          d_out_appl <= a;
          payload_count := payload_count - 1;
          wait until (clk'event and clk = '1');
          sop_out_appl <= '0';
          
          while payload_count > 1 loop
            readline(in_file, line_in);
            hread(line_in, a);
            d_out_appl <= a;
            payload_count := payload_count - 1;
            wait until (clk'event and clk = '1');
          end loop;
          
          readline(in_file, line_in);
          hread(line_in, a);
          d_out_appl <= a;
          eof_out_appl <= '1';
          wait until (clk'event and clk = '1');
          eof_out_appl <= '0';
--          dataEn_out_appl <= '0';
          
        elsif payload_count = 1 then
          readline(in_file, line_in);
          hread(line_in, a);
          d_out_appl <= a;
          dataEn_out_appl <= '1';
          sop_out_appl <= '1';
          eof_out_appl <= '1';
          wait until (clk'event and clk = '1');
--          dataEn_out_appl <= '0';
          sop_out_appl <= '0';
          eof_out_appl <= '0';
        end if;

        
        d_out_appl <= AAL5_trailer_1;
        wait until (clk'event and clk = '1');
        d_out_appl <= AAL5_trailer_2;
      end if;
        
    END IF;
    assert not endfile(in_file) report "Last Line Read" severity NOTE;
  END PROCESS read_process;
  
END behavioral;
